use sprinklers_core::section_interface::{MockSectionInterface, SecId, SectionInterface};

#[cfg(feature = "sprinklers_linux")]
use sprinklers_linux::LinuxGpioConfig;

use serde::{Deserialize, Serialize};
use std::sync::Arc;

#[derive(Debug, Serialize, Deserialize)]
#[serde(tag = "provider")]
pub enum SectionInterfaceConfig {
    Mock {
        num_sections: SecId,
    },
    #[cfg(feature = "sprinklers_linux")]
    LinuxGpio(LinuxGpioConfig),
}

impl Default for SectionInterfaceConfig {
    fn default() -> Self {
        SectionInterfaceConfig::Mock { num_sections: 6 }
    }
}

impl SectionInterfaceConfig {
    pub fn build(self) -> eyre::Result<Arc<dyn SectionInterface>> {
        Ok(match self {
            SectionInterfaceConfig::Mock { num_sections } => {
                Arc::new(MockSectionInterface::new(num_sections))
            }
            #[cfg(feature = "sprinklers_linux")]
            SectionInterfaceConfig::LinuxGpio(config) => {
                Arc::new(config.build()?)
            }
        })
    }
}
